OutputSlice

Struct OutputSlice 

pub struct OutputSlice<'a> {
    pub obuf: &'a mut [u8],
    pub pos: usize,
}
A non-allocating append sink backed by a caller-provided slice.

Its logical view is the prefix written so far (given by pos).

Fields

obuf: &'a mut [u8]pos: usize

Implementations

Source

impl<'a> OutputSlice<'a>

pub open spec fn final_destination(&self) -> Seq<u8>

{ final(self.obuf)@ }

The prophetic final contents of the caller-provided backing slice.

pub exec fn new(obuf: &'a mut [u8]) -> output : Self

ensures
output@ == Seq::empty(),
output.fits(old(obuf)@.len()),
forall |len: nat| #[trigger] output.fits(len) == (len <= old(obuf)@.len()),
output.final_destination() == final(obuf)@,

Creates an empty logical output over obuf without allocating.